// MARK: Types

struct Validator<T> {
    let validatorFailureType: Any.Type
}

protocol ValidatorType {
    associatedtype Data
    associatedtype Failure
    func validator() -> Validator<Data>
}

extension ValidatorType {
    func validator() -> Validator<Data> {
        .init(validatorFailureType: Failure.self)
    }
}

// MARK: Failing example

extension Validator where T == String {
    struct V: ValidatorType {
        typealias Data = T // or String
        struct Failure {}
    }
}

Validator.V().validator() // failed to demangle witness for associated type 'Failure' in conformance '(extension in __lldb_expr_1):__lldb_expr_1.Validator<Swift.String>.V4: ValidatorType' from mangled name '13__lldb_expr_19ValidatorVAASSRszrlE2V4V7FailureVyx__G'

// MARK: Passing examples

struct V1: ValidatorType {
    typealias Data = String
    struct Failure {}
}
V1().validator() // works

extension Validator {
    struct V2: ValidatorType {
        typealias Data = T
        struct Failure {}
    }
}
Validator<String>.V2().validator() // works

protocol SomeProtocol {}
extension String: SomeProtocol {}

extension Validator where T: SomeProtocol {
    struct V3: ValidatorType {
        typealias Data = String
        struct Failure {}
    }
}

Validator<String>.V3().validator() // works fine


